/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.vse.keg.ontologymanager;

import gate.Annotation;
import gate.AnnotationSet;
import gate.Document;
import gate.FeatureMap;
import gate.Node;
import gate.util.InvalidOffsetException;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Milan Dojchinovski
 */
public class TextProcessor {
    private static TextProcessor instance = null;
    public static TextProcessor getInstance(String ontologyLocation){
    
        if(instance == null){
            instance = new TextProcessor();
            DBpediaOntologyManager.setOntologyLocation(ontologyLocation);
        }
        return instance;
    }
    
    
    public void processDocument(Document doc, String gt_as_label, String thd_as_label, String gt_ann_name, String thd_ann_name) {
        
        AnnotationSet thdAnnotSet = doc.getAnnotations(thd_as_label);
        AnnotationSet gtAnnotSet = doc.getAnnotations(gt_as_label);
        
        AnnotationSet thdAnn = thdAnnotSet.get(thd_ann_name);
        
        Iterator thdAnnIter = thdAnn.iterator();
        
        while(thdAnnIter.hasNext()) {
            
            try {
            
                Annotation thd_ann = (gate.Annotation) thdAnnIter.next();
                Node annStart = thd_ann.getStartNode();
                Node annEnd = thd_ann.getEndNode();
                
                AnnotationSet gt_ann_set = gtAnnotSet.get(annStart.getOffset(), annEnd.getOffset()).get(gt_ann_name);
                Iterator gtAnnIter = gt_ann_set.iterator();
                
                while(gtAnnIter.hasNext()) {
                    
                    Annotation gt_ann = (gate.Annotation) gtAnnIter.next();
                    
                    Long thd_start = thd_ann.getStartNode().getOffset();
                    Long thd_end = thd_ann.getEndNode().getOffset();
                    Long gt_start = gt_ann.getStartNode().getOffset();
                    Long gt_end = gt_ann.getEndNode().getOffset();
                    
                    // check if gt and thd annotation overlap
                    if(thd_start.equals(gt_start) && thd_end.equals(gt_end)) {
                        
                        FeatureMap thdFeatures = thd_ann.getFeatures();
                        FeatureMap gtFeatures = gt_ann.getFeatures();
                        
                        Object thdTypeURIObj = thdFeatures.get("typeURI");
                        Object gtTypeURIObj = gtFeatures.get("typeURI");
                        String thdTypeURI = null;
                        String gtTypeURI = null;
                        
                        if(thdTypeURIObj != null){
                            thdTypeURI = thdTypeURIObj.toString();
                        }
                        if(gtTypeURIObj != null){
                            gtTypeURI = gtTypeURIObj.toString();
                        }
                        
                        if(gtTypeURI != null && thdTypeURI != null) {
                        
                            if(thdTypeURI.startsWith("http://dbpedia.org/ontology/")) {

                                int dist = DBpediaOntologyManager.getInstance().doSuperclassComparison(thdTypeURI, gtTypeURI);

                                if(dist == 0) {
                                    // exact match
                                    thdFeatures.put("matchtype", "exactmatch");
                                    thdFeatures.put("distance", "0");
                                    thdFeatures.put("thd_aligned", thdTypeURI);
                                    gtFeatures.put("matchtype", "exactmatch");
                                    gtFeatures.put("distance", "0");
                                    gtFeatures.put("thd_aligned", thdTypeURI);
                                } else if(dist == -1) {
                                    // no match, try subclass match
                                   dist = DBpediaOntologyManager.getInstance().doSuperclassComparison(gtTypeURI, thdTypeURI);

                                   if(dist == -1) {
                                       // no match for subclass
                                       thdFeatures.put("matchtype", "nomatch");
                                       gtFeatures.put("matchtype", "nomatch");
                                   } else {
                                       // matched subclass
                                       thdFeatures.put("matchtype", "submatch");
                                       thdFeatures.put("distance", dist);
                                       thdFeatures.put("thd_aligned", thdTypeURI);
                                       gtFeatures.put("matchtype", "submatch");
                                       gtFeatures.put("distance", dist);
                                       gtFeatures.put("thd_aligned", thdTypeURI);
                                   }
                                } else {
                                    // matched superclass
                                    thdFeatures.put("matchtype", "supertype");
                                    thdFeatures.put("distance", dist);
                                    thdFeatures.put("thd_aligned", thdTypeURI);
                                    gtFeatures.put("matchtype", "supertype");
                                    gtFeatures.put("distance", dist);
                                    gtFeatures.put("thd_aligned", thdTypeURI);
                                }
                            } else {
                                // type is not from DBpedia ontology -> no match
                                thdFeatures.put("matchtype", "nomatch");                        
                                gtFeatures.put("matchtype", "nomatch");                        
                            }
                            thd_ann.setFeatures(thdFeatures);
                            gt_ann.setFeatures(gtFeatures);
                        }
                    }
                }
            } catch (Exception ex) {            
                Logger.getLogger(TextProcessor.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
}
